@@ -14,7 +14,7 @@ use File::Glob qw(:bsd_glob);
 use POSIX qw(strftime);                                                         # http://www.cplusplus.com/reference/ctime/strftime/
 use Data::Dump qw(dump);
 
-our $VERSION = '2017.222';
+our $VERSION = '2017.313';
 
 &saveToS3('DataTableText') if 0;                                                # Save code
 
@@ -45,15 +45,44 @@ sub fileSize($)
   (stat($file))[7]
  }
 
+#-------------------------------------------------------------------------------
+# Create a file path from file name components
+#-------------------------------------------------------------------------------
+
+sub filePath(@)                                                                 # File
+ {my (@file) = @_;
+  my $s = '';
+  my $t = join '/', map {s/[\/\\]+\Z//r} @_;
+  $t
+ }
+
+sub filePathDir(@)                                                              # Directory
+ {my $f = filePath(@_);
+  $f =~ s/[\/\\]+\Z//s;
+  $f.'/';
+ }
+
+sub quoteFile($)                                                                # Quote a file name
+ {my ($f) = @_;
+  "\"$f\""
+ }
+
+sub currentDirectory()                                                          # Directory
+ {my $f = qx(pwd);
+  chomp($f);
+  $f.'/'
+ }
+
 #-------------------------------------------------------------------------------
 # Make a path
 #-------------------------------------------------------------------------------
 
 sub makePath($)
  {my ($path) = @_;
-  my @p = split //, $path; pop @p while @p and $p[-1] !~ /[\\\/]/;
-  my $p = join '', @p;
-  return 0 if -d $p;
+  my @p = split /[\\\/]+/, $path; pop @p;
+  return 1 unless @p;
+  my $p = join '/', @p;
+  return 2 if -d $p;
   eval {make_path($p)};
   -d $p or confess "Cannot makePath $p";
   1
@@ -112,6 +141,22 @@ sub writeFile($$)
   -e $file or confess "Failed to write to file $file";
  }
 
+#-------------------------------------------------------------------------------
+# Append file
+#-------------------------------------------------------------------------------
+
+sub appendFile($$)
+ {my ($file, $string) = @_;
+  $file or confess "No file name supplied";
+  $string or carp "No string for file $file";
+  makePath($file);
+  open my $F, ">>$file" or confess "Cannot open $file for write";
+  binmode($F, ":utf8");
+  print  {$F} $string;
+  close  ($F);
+  -e $file or confess "Failed to write to file $file";
+ }
+
 #-------------------------------------------------------------------------------
 # Write binary file
 #-------------------------------------------------------------------------------
@@ -128,6 +173,62 @@ sub writeBinaryFile($$)
   -e $file or confess "Failed to write in binary to file $file";
  }
 
+#-------------------------------------------------------------------------------
+# Dimensions of an image via ImageMagick identify
+#-------------------------------------------------------------------------------
+
+sub imageSize($)                                                                # Return (width, height) of an image
+ {my ($image) = @_;
+  -e $image or confess "Cannot get size of image $image as file does not exist";
+  my $s = qx(identify -verbose "$image");
+  if ($s =~ /Geometry: (\d+)x(\d+)/s)
+   {return ($1, $2);
+   }
+  else
+   {confess "Cannot get image size for $image from:\n$s";
+   }
+ }
+
+#-------------------------------------------------------------------------------
+# Convert an image to Jpx format
+#-------------------------------------------------------------------------------
+
+sub convertImageToJpx($$$)                                                      # Convert an image to jpx format
+ {my ($source, $target, $size) = @_;                                            # Source file, target folder (as multiple files will be created),  size of each tile
+  -e $source or confess "convertImageJpx: cannot convert image file $source, as file does not exist";
+  makePath($target);
+  my ($w, $h) = imageSize($source);
+  writeFile(filePath($target, "jpx.data"), <<END);
+version 1
+type    jpx
+size    $size
+source  $source
+width   $w
+height  $h
+END
+
+  if (1)
+   {my $s = quoteFile($source);
+    my $t = quoteFile($target);
+    qx(convert $s -crop ${size}x${size} $t);
+   }
+
+  if (1)
+   {my $W = int($w/$size); ++$W if $w % $size;
+    my $H = int($h/$size); ++$H if $h % $size;
+    my $k = 0;
+    for   my $Y(1..$H)
+     {for my $X(1..$W)
+       {my $s = "${target}-$k";
+        my $t = "${target}/${Y}_${X}.jpg";
+        rename $s, $t or confess "Cannot rename $s to $t";
+        -e $t or confess "Cannot create $t";
+        ++$k;
+       }
+     }
+   }
+ }
+
 #-------------------------------------------------------------------------------
 # Tabularize text - basic version
 #-------------------------------------------------------------------------------
@@ -509,12 +610,15 @@ use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @ISA          = qw(Exporter);
 @EXPORT       = qw(formatTable);
-@EXPORT_OK    = qw(checkKeys containingPowerOfTwo dateStamp dateTimeStamp
-extractDocumentation fileList fileSize formatTableBasic
-genLValueArrayMethods genLValueHashMethods genLValueScalarMethods genLValueScalarMethodsWithDefaultValues
-indentString
-loadArrayArrayFromLines loadArrayFromLines  loadHashArrayFromLines loadHashFromLines
-makePath nws pad powerOfTwo
+@EXPORT_OK    = qw(appendFile checkKeys containingPowerOfTwo currentDirectory
+dateStamp dateTimeStamp
+extractDocumentation fileList filePath filePathDir fileSize formatTableBasic
+genLValueArrayMethods genLValueHashMethods
+genLValueScalarMethods genLValueScalarMethodsWithDefaultValues
+imageSize indentString
+loadArrayArrayFromLines loadArrayFromLines
+loadHashArrayFromLines loadHashFromLines
+makePath nws pad powerOfTwo quoteFile
 readBinaryFile readFile saveToS3 timeStamp trim writeBinaryFile writeFile);
 %EXPORT_TAGS  = (all=>[@EXPORT, @EXPORT_OK]);
 
@@ -626,7 +730,12 @@ modified under the same terms as Perl itself.
 =cut
 
 __DATA__
-use Test::More tests => 45;
+use Test::More tests => 47;
+if (1)                                                                          # File paths
+ {ok filePath   (qw(/aaa bbb ccc ddd.eee)) eq "/aaa/bbb/ccc/ddd.eee";
+  ok filePathDir(qw(/aaa bbb ccc ddd))     eq "/aaa/bbb/ccc/ddd/";
+ }
+
 if (1)                                                                          # Unicode
  {use utf8;
   my $z = "𝝰𝝱𝝲";
